/*
© Copyright 2005 Apple Computer, Inc. All rights reserved.

IMPORTANT:  This Apple software and the associated images located in
/System/Library/WidgetResources/AppleClasses/ (collectively "Apple Software")
are supplied to you by Apple Computer, Inc. (“Apple”) in consideration of your
agreement to the following terms. Your use, installation and/or redistribution
of this Apple Software constitutes acceptance of these terms. If you do not
agree with these terms, please do not use, install, or redistribute this Apple
Software.

In consideration of your agreement to abide by the following terms, and subject
to these terms, Apple grants you a personal, non-exclusive license, under
Apple’s copyrights in the Apple Software, to use, reproduce, and redistribute
the Apple Software, in text form (for JavaScript files) or binary form (for
associated images), for the sole purpose of creating Dashboard widgets for Mac
OS X.

If you redistribute the Apple Software, you must retain this entire notice and
the warranty disclaimers and limitation of liability provisions (last two
paragraphs below) in all such redistributions of the Apple Software.

You may not use the name, trademarks, service marks or logos of Apple to endorse
or promote products that include the Apple Software without the prior written
permission of Apple. Except as expressly stated in this notice, no other rights
or licenses, express or implied, are granted by Apple herein, including but not
limited to any patent rights that may be infringed by your products that
incorporate the Apple Software or by other works in which the Apple Software may
be incorporated.

The Apple Software is provided on an "AS IS" basis.  APPLE MAKES NO WARRANTIES,
EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF
NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
REGARDING THE APPPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN COMBINATION
WITH YOUR PRODUCTS.

IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, AND/OR DISTRIBUTION OF THE
APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

function CreateHorizontalLevelIndicator(levelIndicatorID, value, minValue, maxValue, warningValue, criticalValue, spacing, stacked, interactive, continuous, imageOn, imageOff, imageWarning, imageCritical, imageWidth, imageHeight, enabled, onchanged)
{
	var levelIndicatorElement = document.getElementById(levelIndicatorID);
	if (!levelIndicatorElement.loaded) 
	{
		levelIndicatorElement.loaded = true;
		try { onchanged = eval(onchanged); } catch (e) { onchanged = null; }
		levelIndicatorElement.object = new HorizontalLevelIndicator(levelIndicatorElement, value, minValue, maxValue, warningValue, criticalValue, spacing, stacked, interactive, continuous, imageOn, imageOff, imageWarning, imageCritical, imageWidth, imageHeight, onchanged);
		return levelIndicatorElement.object;
	}
}

function CreateVerticalLevelIndicator(levelIndicatorID, value, minValue, maxValue, warningValue, criticalValue, spacing, stacked, interactive, continuous, imageOn, imageOff, imageWarning, imageCritical, imageWidth, imageHeight, enabled, onchanged)
{
	var levelIndicatorElement = document.getElementById(levelIndicatorID);
	if (!levelIndicatorElement.loaded) 
	{
		levelIndicatorElement.loaded = true;
		try { onchanged = eval(onchanged); } catch (e) { onchanged = null; }
		levelIndicatorElement.object = new VerticalLevelIndicator(levelIndicatorElement, value, minValue, maxValue, warningValue, criticalValue, spacing, stacked, interactive, continuous, imageOn, imageOff, imageWarning, imageCritical, imageWidth, imageHeight, onchanged);
		return levelIndicatorElement.object;
	}
}

function LevelIndicator(levelIndicator, value, minValue, maxValue, warningValue, criticalValue, spacing, stacked, interactive, continuous, imageOff, imageOn, imageWarning, imageCritical, imageWidth, imageHeight, onchanged)
{
}

LevelIndicator.prototype._init = function(value, minValue, maxValue, warningValue, criticalValue, spacing, stacked, interactive, imageOn, imageOff, imageWarning, imageCritical, imageWidth, imageHeight)
{
	// For JavaScript event handlers
	var _self = this;
	
	this.minValue = minValue;
	this.maxValue = maxValue;
	this.warningValue = warningValue;
	this.criticalValue = criticalValue;
	this.spacing = spacing;
	this.stacked = stacked;
	this.interactive = interactive;

	this._captureEventHandler = function(event) { _self._captureEvent(event); };
	this._mousedownTrackHandler = function(event) { _self._mousedownTrack(event); };
	this._mousemoveTrackHandler = function(event) { _self._mousemoveTrack(event); };
	this._mouseupTrackHandler = function(event) { _self._mouseupTrack(event); };
	
	var style = null;
	var element = null;

	// Level Indicator Track
	element = document.createElement("div");
	style = element.style;
	style.appleDashboardRegion = "dashboard-region(control rectangle)";
	style.height = "100%";
	style.width = "100%";
	this._levelIndicator.appendChild(element);
	this._track = element;
			
	// Add event listeners
	this._track.addEventListener("mousedown", this._mousedownTrackHandler, true);
	
	this.refresh();
}


LevelIndicator.prototype.remove = function()
{
	var parent = this._track.parentNode;
	parent.removeChild(this._track);
}

/*
 * refresh() member function
 * Refresh the current level indicator position.
 * Call this to make the level indicator appear after the widget has loaded and 
 * the LevelIndicator object has been instantiated.
 */
LevelIndicator.prototype.refresh = function()
{	
	this._computedLevelIndicatorStyle = document.defaultView.getComputedStyle(this._levelIndicator, '');
	this._setValueTo(this.value);
}

LevelIndicator.prototype._setValueTo = function(newValue)
{	
	this.value = newValue;
	
	// Remove the existing children
	var track = this._track;
	while (track.hasChildNodes())
		track.removeChild(track.firstChild);
	
	this._layoutElements();

	if (this.continuous && this.onchanged != null)
		this.onchanged(this.value);
}

// Capture events that we don't handle but also don't want getting through
LevelIndicator.prototype._captureEvent = function(event)
{
	event.stopPropagation();
	event.preventDefault();
}

LevelIndicator.prototype._mousedownTrack = function(event)
{	
	// temporary event listeners
	document.addEventListener("mousemove", this._mousemoveTrackHandler, true);
	document.addEventListener("mouseup", this._mouseupTrackHandler, true);
	document.addEventListener("mouseover", this._captureEventHandler, true);
	document.addEventListener("mouseout", this._captureEventHandler, true);
		
	this._setValueTo(this._computeValueFromMouseEvent(event));
} 

LevelIndicator.prototype._mousemoveTrack = function(event)
{	
	this._setValueTo(this._computeValueFromMouseEvent(event));
} 

LevelIndicator.prototype._mouseupTrack = function(event)
{	
	document.removeEventListener("mousemove", this._mousemoveTrackHandler, true);
	document.removeEventListener("mouseup", this._mouseupTrackHandler, true);
	document.removeEventListener("mouseover", this._captureEventHandler, true);
	document.removeEventListener("mouseout", this._captureEventHandler, true);

	// Fire our onchanged event now if they have discontinuous event firing
	if (!this.continuous && this.onchanged != null)
		this.onchanged(this.value);
} 

LevelIndicator.prototype.setValue = function(newValue)
{
	this.value = newValue;
	this.refresh();
}

LevelIndicator.prototype.setMinValue = function(newValue)
{
	this.minValue = newValue;
	this.refresh();
}

LevelIndicator.prototype.setMaxValue = function(newValue)
{
	this.maxValue = newValue;
	this.refresh();
}

LevelIndicator.prototype.setWarningValue = function(newValue)
{
	this.warningValue = newValue;
	this.refresh();
}

LevelIndicator.prototype.setCriticalValue = function(newValue)
{
	this.criticalValue = newValue;
	this.refresh();
}

LevelIndicator.prototype.setSpacing = function(newValue)
{
	this.spacing = newValue;
	this.refresh();
}

LevelIndicator.prototype.setStacked = function(newValue)
{
	this.stacked = newValue;
	this.refresh();
}

LevelIndicator.prototype.setInteractive = function(newValue)
{
	this.interactive = newValue;

	if (this.interactive)
		this._track.style.appleDashboardRegion = "dashboard-region(control rectangle)";
	else
		this._track.style.appleDashboardRegion = null;
		
	this.refresh();
}

LevelIndicator.prototype.setImageOff = function(newValue)
{
	this.imageOffPath = newValue;	
	this.refresh();
}

LevelIndicator.prototype.setImageOn = function(newValue)
{
	this.imageOnPath = newValue;	
	this.refresh();
}

LevelIndicator.prototype.setImageWarning = function(newValue)
{
	this.imageWarningPath = newValue;	
	this.refresh();
}

LevelIndicator.prototype.setImageCritical = function(newValue)
{
	this.imageCriticalPath = newValue;	
	this.refresh();
}

LevelIndicator.prototype.setImageWidth = function(newValue)
{
	this.imageWidth = newValue;	
	this.refresh();
}

LevelIndicator.prototype.setImageHeight = function(newValue)
{
	this.imageHeight = newValue;	
	this.refresh();
}

/*******************************************************************************
* HorizontalLevelIndicator
* Implementation of LevelIndicator
*
*
*/

function HorizontalLevelIndicator(levelIndicator, value, minValue, maxValue, warningValue, criticalValue, spacing, stacked, interactive, continuous, imageOff, imageOn, imageWarning, imageCritical, imageWidth, imageHeight, onchanged)
{
	/* Objects */
	this._levelIndicator = levelIndicator;
	
	/* public properties */
	// These are read-write. Set them as needed.
	this.onchanged = onchanged;
	this.continuous = continuous; // Fire onchanged live, as opposed to onmouseup
	
	// These are read-only. Use the setter functions to set them.
	this.value = value;
	
	/* Internal objects */
	this._track = null;
		
	this.imageWidth = imageWidth;
	this.imageHeight = imageHeight;
	this.imageOffPath = imageOff == null ? "Images/HorizontalOff.png" : imageOff;
	this.imageOnPath = imageOn == null ? "Images/HorizontalOn.png" : imageOn;
	this.imageWarningPath = imageWarning == null ? "Images/HorizontalWarning.png" : imageWarning;
	this.imageCriticalPath = imageCritical == null ? "Images/HorizontalCritical.png" : imageCritical;
		
	this._init(value, minValue, maxValue, warningValue, criticalValue, spacing, stacked, interactive, imageOn, imageOff, imageWarning, imageCritical, imageWidth, imageHeight);
}

// Inherit from LevelIndicator
HorizontalLevelIndicator.prototype = new LevelIndicator(null);

HorizontalLevelIndicator.prototype._getMousePosition = function(event)
{
	if (event !== undefined)
		return event.x;
	else
		return 0;
}

HorizontalLevelIndicator.prototype._computeValueFromMouseEvent = function(event)
{
	var style = this._computedLevelIndicatorStyle;
	var left = style ? parseInt(style.getPropertyValue("left"), 10) : 0;
	var width = style ? parseInt(style.getPropertyValue("width"), 10) : 0;
	var position = this._getMousePosition(event);
	var newValue = this.minValue + (((this.maxValue - this.minValue) * (position - left)) / width);
	
	if (newValue < this.minValue)
		newValue = this.minValue;
	else if (newValue > this.maxValue)
		newValue = this.maxValue;
		
	return newValue;
}

HorizontalLevelIndicator.prototype._computePositionFromValue = function(newValue)
{
	var style = this._computedLevelIndicatorStyle;
	var left = style ? parseInt(style.getPropertyValue("left"), 10) : 0;
	var width = style ? parseInt(style.getPropertyValue("width"), 10) : 0;
	var position = (width * (newValue - this.minValue)) / (this.maxValue - this.minValue);
	
	return position;
}

HorizontalLevelIndicator.prototype._computeValueFromPosition = function(position)
{
	var style = this._computedLevelIndicatorStyle;
	var left = style ? parseInt(style.getPropertyValue("left"), 10) : 0;
	var width = style ? parseInt(style.getPropertyValue("width"), 10) : 0;
	var newValue = this.minValue + (((this.maxValue - this.minValue) * (position - left)) / width);
		
	return newValue;
}

HorizontalLevelIndicator.prototype._computeLevelIndicatorLength = function()
{
	// get the current actual slider length
	var style = this._computedLevelIndicatorStyle;
	return style ? parseInt(style.getPropertyValue("width"), 10) : 0;
}

HorizontalLevelIndicator.prototype._layoutElements = function()
{
	var length = this._computeLevelIndicatorLength();
	var valuePosition = this._computePositionFromValue(this.value);
	var delta = 0;
	
	var imagePath = null;
	
	while (delta + this.imageWidth <= length)
	{
		var element = document.createElement("div");
		var style = element.style;
		style.position = "absolute";
		style.display = "block";
		style.top = "0px";
		style.left = delta + "px";
		style.width = this.imageWidth + "px";
		style.height = this.imageHeight + "px";
		
		if (this.stacked)
		{
			var currentValue = this._computeValueFromPosition(delta + this.imageWidth);
			
			if (delta >= valuePosition)
				imagePath = this.imageOffPath;
			else if (currentValue >= this.criticalValue)
				imagePath = this.imageCriticalPath;
			else if (currentValue >= this.warningValue)
				imagePath = this.imageWarningPath;
			else
				imagePath = this.imageOnPath;
		}
		else
		{
			if (delta >= valuePosition)
				imagePath = this.imageOffPath;
			else if (this.value >= this.criticalValue)
				imagePath = this.imageCriticalPath;
			else if (this.value >= this.warningValue)
				imagePath = this.imageWarningPath;
			else
				imagePath = this.imageOnPath;
		}

		style.background = "url(" + imagePath + ") no-repeat top left";
		this._track.appendChild(element);
					
		delta += this.imageWidth + this.spacing;
	}
}

/*******************************************************************************
* VerticalLevelIndicator
* Implementation of LevelIndicator
*
*
*/

function VerticalLevelIndicator(levelIndicator, value, minValue, maxValue, warningValue, criticalValue, spacing, stacked, interactive, continuous, imageOff, imageOn, imageWarning, imageCritical, imageWidth, imageHeight, onchanged)
{
	/* Objects */
	this._levelIndicator = levelIndicator;
	
	/* public properties */
	// These are read-write. Set them as needed.
	this.onchanged = onchanged;
	this.continuous = continuous; // Fire onchanged live, as opposed to onmouseup
	
	// These are read-only. Use the setter functions to set them.
	this.value = value;
	
	/* Internal objects */
	this._track = null;
		
	this.imageWidth = imageWidth;
	this.imageHeight = imageHeight;
	this.imageOffPath = imageOff == null ? "Images/VerticalOff.png" : imageOff;
	this.imageOnPath = imageOn == null ? "Images/VerticalOn.png" : imageOn;
	this.imageWarningPath = imageWarning == null ? "Images/VerticalWarning.png" : imageWarning;
	this.imageCriticalPath = imageCritical == null ? "Images/VerticalCritical.png" : imageCritical;

	this._init(value, minValue, maxValue, warningValue, criticalValue, spacing, stacked, interactive, imageOn, imageOff, imageWarning, imageCritical, imageWidth, imageHeight);
}

// Inherit from LevelIndicator
VerticalLevelIndicator.prototype = new LevelIndicator(null);

VerticalLevelIndicator.prototype._getMousePosition = function(event)
{
	if (event !== undefined)
		return event.y;
	else
		return 0;
}

VerticalLevelIndicator.prototype._computeValueFromMouseEvent = function(event)
{
	var style = this._computedLevelIndicatorStyle;
	var top = style ? parseInt(style.getPropertyValue("top"), 10) : 0;
	var height = style ? parseInt(style.getPropertyValue("height"), 10) : 0;
	var position = this._getMousePosition(event);
	var newValue = this.minValue + (((this.maxValue - this.minValue) * ((top + height) - position)) / height);
	
	if (newValue < this.minValue)
		newValue = this.minValue;
	else if (newValue > this.maxValue)
		newValue = this.maxValue;

	return newValue;
}

VerticalLevelIndicator.prototype._computePositionFromValue = function(newValue)
{
	var style = this._computedLevelIndicatorStyle;
	var top = style ? parseInt(style.getPropertyValue("top"), 10) : 0;
	var height = style ? parseInt(style.getPropertyValue("height"), 10) : 0;
	var position = (height * (newValue - this.minValue)) / (this.maxValue - this.minValue);
	
	return height - position;
}

VerticalLevelIndicator.prototype._computeValueFromPosition = function(position)
{
	var style = this._computedLevelIndicatorStyle;
	var top = style ? parseInt(style.getPropertyValue("top"), 10) : 0;
	var height = style ? parseInt(style.getPropertyValue("height"), 10) : 0;
	var newValue = this.minValue + (((this.maxValue - this.minValue) * ((top + height) - position)) / height);
	
	return newValue;
}

VerticalLevelIndicator.prototype._computeLevelIndicatorLength = function()
{
	// get the current actual slider length
	var style = this._computedLevelIndicatorStyle;
	return style ? parseInt(style.getPropertyValue("height"), 10) : 0;
}

VerticalLevelIndicator.prototype._layoutElements = function()
{
	var length = this._computeLevelIndicatorLength();
	var valuePosition = this._computePositionFromValue(this.value);
	var delta = length - this.imageHeight;
	
	var imagePath = null;

	while (delta >= 0)
	{
		var element = document.createElement("div");
		var style = element.style;
		style.position = "absolute";
		style.display = "block";
		style.top = delta + "px";
		style.left = "0px";
		style.width = this.imageWidth + "px";
		style.height = this.imageHeight + "px";
		
		if (this.stacked)
		{
			var currentValue = this._computeValueFromPosition(delta + this.imageHeight);

			if (delta + this.imageHeight <= valuePosition)
				imagePath = this.imageOffPath;
			else if (currentValue >= this.criticalValue)
				imagePath = this.imageCriticalPath;
			else if (currentValue >= this.warningValue)
				imagePath = this.imageWarningPath;
			else 
				imagePath = this.imageOnPath;
		}
		else
		{
			if (delta + this.imageHeight <= valuePosition)
				imagePath = this.imageOffPath;
			else if (this.value >= this.criticalValue)
				imagePath = this.imageCriticalPath;
			else if (this.value >= this.warningValue)
				imagePath = this.imageWarningPath;
			else 
				imagePath = this.imageOnPath;
		}

		style.background = "url(" + imagePath + ") no-repeat top left";
		this._track.appendChild(element);
					
		delta -= this.imageHeight + this.spacing;
	}
}